/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
/* Copyright (C) 2018 Jernej Skrabec <jernej.skrabec@siol.net> */

#ifndef _DT_BINDINGS_CLOCK_SUN8I_TCON_TOP_H_
#define _DT_BINDINGS_CLOCK_SUN8I_TCON_TOP_H_

#define CLK_TCON_TOP_TV0	0
#define CLK_TCON_TOP_TV1	1
#define CLK_TCON_TOP_DSI	2

#endif /* _DT_BINDINGS_CLOCK_SUN8I_TCON_TOP_H_ */
