/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Delta TN48M CPLD GPIO driver
 *
 * Copyright (C) 2021 Sartura Ltd.
 *
 * Author: Robert Marko <robert.marko@sartura.hr>
 */

#ifndef _DT_BINDINGS_RESET_TN48M_H
#define _DT_BINDINGS_RESET_TN48M_H

#define CPU_88F7040_RESET	0
#define CPU_88F6820_RESET	1
#define MAC_98DX3265_RESET	2
#define PHY_88E1680_RESET	3
#define PHY_88E1512_RESET	4
#define POE_RESET		5

#endif /* _DT_BINDINGS_RESET_TN48M_H */
