/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2021 MediaTek Inc.
 */

#ifndef __LINUX_REGULATOR_MT6359_H
#define __LINUX_REGULATOR_MT6359_H

enum {
	MT6359_ID_VS1 = 0,
	MT6359_ID_VGPU11,
	MT6359_ID_VMODEM,
	MT6359_ID_VPU,
	MT6359_ID_VCORE,
	MT6359_ID_VS2,
	MT6359_ID_VPA,
	MT6359_ID_VPROC2,
	MT6359_ID_VPROC1,
	MT6359_ID_VCORE_SSHUB,
	MT6359_ID_VGPU11_SSHUB = MT6359_ID_VCORE_SSHUB,
	MT6359_ID_VAUD18 = 10,
	MT6359_ID_VSIM1,
	MT6359_ID_VIBR,
	MT6359_ID_VRF12,
	MT6359_ID_VUSB,
	MT6359_ID_VSRAM_PROC2,
	MT6359_ID_VIO18,
	MT6359_ID_VCAMIO,
	MT6359_ID_VCN18,
	MT6359_ID_VFE28,
	MT6359_ID_VCN13,
	MT6359_ID_VCN33_1,
	MT6359_ID_VAUX18,
	MT6359_ID_VSRAM_OTHERS,
	MT6359_ID_VEFUSE,
	MT6359_ID_VXO22,
	MT6359_ID_VRFCK,
	MT6359_ID_VBIF28,
	MT6359_ID_VIO28,
	MT6359_ID_VEMC,
	MT6359_ID_VCN33_2,
	MT6359_ID_VA12,
	MT6359_ID_VA09,
	MT6359_ID_VRF18,
	MT6359_ID_VSRAM_MD,
	MT6359_ID_VUFS,
	MT6359_ID_VM18,
	MT6359_ID_VBBCK,
	MT6359_ID_VSRAM_PROC1,
	MT6359_ID_VSIM2,
	MT6359_ID_VSRAM_OTHERS_SSHUB,
	MT6359_ID_VCN33_1_BT,
	MT6359_ID_VCN33_1_WIFI,
	MT6359_ID_VCN33_2_BT,
	MT6359_ID_VCN33_2_WIFI,
	MT6359_ID_RG_MAX,
};

#define MT6359_MAX_REGULATOR	MT6359_ID_RG_MAX

#endif /* __LINUX_REGULATOR_MT6359_H */
