/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * AD5504 SPI DAC driver
 *
 * Copyright 2011 Analog Devices Inc.
 */

#ifndef SPI_AD5504_H_
#define SPI_AD5504_H_

struct ad5504_platform_data {
	u16				vref_mv;
};

#endif /* SPI_AD5504_H_ */
