/* SPDX-License-Identifier: GPL-2.0-or-later OR MIT */

#ifndef __DT_BINDINGS_SOC_BCM_PMB_H
#define __DT_BINDINGS_SOC_BCM_PMB_H

#define BCM_PMB_PCIE0				0x01
#define BCM_PMB_PCIE1				0x02
#define BCM_PMB_PCIE2				0x03
#define BCM_PMB_HOST_USB			0x04
#define BCM_PMB_SATA				0x05

#endif
