/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (C) 2022 Alibaba Group Holding Limited.
 * Copyright (c) 2024 Samsung Electronics Co., Ltd.
 * Author: Michal Wilczynski <m.wilczynski@samsung.com>
 */

#ifndef __DT_BINDINGS_POWER_TH1520_H
#define __DT_BINDINGS_POWER_TH1520_H

#define TH1520_AUDIO_PD		0
#define TH1520_VDEC_PD		1
#define TH1520_NPU_PD		2
#define TH1520_VENC_PD		3
#define TH1520_GPU_PD		4
#define TH1520_DSP0_PD		5
#define TH1520_DSP1_PD		6

#endif
